/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __PXE_PREBOOT_H__
#define __PXE_PREBOOT_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE data types */
#include "pxe_types.h"

/* Data structure for pxenv_start_unload_stack().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_start_unload_stack_s {
    pxenv_status_t status;
    u_int8_t reserved[10];
} pxenv_unload_stack_t;


/* Data structure for pxenv_get_cached_info().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     pkt_type - the type of packet in the cache. The valid types
 *              are defined below.
 *     buffer_size - the size of the receive buffer in bytes. 
 *     buffer - the buffer address in offset:segment format.
 */
#define PXENV_PACKET_TYPE_DHCP_DISCOVER 1
#define PXENV_PACKET_TYPE_DHCP_ACK      2
#define PXENV_PACKET_TYPE_BINL_REPLY    3

typedef struct pxenv_get_cached_info_s {
    pxenv_status_t status;
    u_int16_t pkt_type;
    u_int16_t buffer_size;
    segoff16_t buffer;
} pxenv_get_cached_info_t;

/* Data structure for pxenv_restart_tftp().
 * parameters:
 *     filename - a NULL terminated string represnting the boot
 *             file name.
 *     buffer_size - the size of the receive buffer in bytes. 
 *     buffer - the buffer address in offset:segment format.
 *     server_ip - the IP address of the TFTP server in Big Endian
 *                 format.
 *     gw_ip - the IP address of gateway in Big Endian
 *             format.
 *     mcast_ip - the multicast IP address in Big Endian format.
 *     tftp_clnt_port - the port the client listens to for a multicast.
 *     tftp_srv_port - the port the server listens to for a multicast.
 *     tftp_open_timeout - the timeout value in seconds used for
 *                         receiving data. If zero, the default of
 *			   TFTP timeout is used.
 *     tftp_reopen_delay - the maximum wait delay time in seconds
 *                         between ACK of a fast packet and a new
 *                         TFTP open.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     packetnum - the packet sequence number.
 *     buffer_size - the size of the packet received in the buffer.
 */
typedef struct pxenv_restart_tftp_s {
    pxenv_status_t status;
    u_int8_t filename[128];
    u_int32_t buffer_size;
    addr32_t buffer;
    ip4_t server_ip;
    ip4_t gw_ip;
    ip4_t mcast_ip;
    udp_port_t tftp_clnt_port;
    udp_port_t tftp_srv_port;
    u_int16_t tftp_open_timeout;
    u_int16_t tftp_reopen_delay;
} pxenv_restart_tftp_t;

/* Data structure for pxenv_start_undi().
 * parameters:
 *     These must contain the same data that was passed to the option
 *     ROM by the system BIOS.
 *     ax - this is the value of the AX register.
 *     bx - this is the value of the BX register.
 *     dx - this is the value of the DX register.
 *     di - this is the value of the DI register.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_start_undi_s {
    pxenv_status_t status;
    u_int16_t ax;
    u_int16_t bx;
    u_int16_t dx;
    u_int16_t di;
    u_int16_t es;
} pxenv_start_undi_t;

/* Data structure for pxenv_stop_undi().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_stop_undi_s {
    pxenv_status_t status;
} pxenv_stop_undi_t;

#define PXENV_UNDI_STARTED 1 /* not even initialized */
#define PXENV_UNDI_INITIALIZED 2 /* initialized and closed (not opened) */
#define PXENV_UNDI_OPENED 3 /* initialized & opened */

/* Data structure for pxenv_start_base().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_start_base_s {
    pxenv_status_t status;
} pxenv_start_base_t;

/* Data structure for pxenv_stop_base().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_stop_base_s {
    pxenv_status_t status;
} pxenv_stop_base_t;

/* Prototypes for the PREBOOT PXE API functions */
pxenv_ret_code_t
pxenv_unload_stack (pxenv_unload_stack_t stackptr);
pxenv_ret_code_t
pxenv_get_cached_info (pxenv_get_cached_info_t cinfoptr);
pxenv_ret_code_t
pxenv_restart_tftp (pxenv_restart_tftp_t session);
pxenv_ret_code_t
pxenv_start_undi (pxenv_start_undi_t startundi);
pxenv_ret_code_t
pxenv_stop_undi (pxenv_stop_undi_t stopundi);
pxenv_ret_code_t
pxenv_start_base (pxenv_start_base_t startbase);
pxenv_ret_code_t
pxenv_stop_base (pxenv_stop_base_t stopbase);
pxenv_ret_code_t

#endif /* __PXE_PREBOOT_H__ */
